@@ -83,11 +83,7 @@ struct MultiviewValues {
 // NOTE: These are the first declarations seen by the compiler
 //       for the API and provide default argument values
 //-------------------------------------------------------------
-std::string ToLower   ( std::string str );
-bool        OnlyDigits( std::string str, bool integerOnly );
-
-std::vector<std::string> SplitString( std::string inString, 
-                                      std::string delimeters );
+std::string ToLower( std::string str );
 
 VectorError ComputeError( std::valarray< double > obs,
                           std::valarray< double > pred );
